@@ -2,13 +2,18 @@
 #' @description \code{bdpsurvival} is used to estimate the survival probability
 #'   (single arm trial; OPC) or hazard ratio (two-arm trial; RCT) for
 #'   right-censored data using the survival analysis implementation of the
-#'   Bayesian discount prior. This code is modeled after
+#'   Bayesian discount prior. In the current implementation, a two-arm analysis
+#'   requires all of current treatment, current control, historical treatment,
+#'   and historical control data. This code is modeled after
 #'   the methodologies developed in Haddad et al. (2017).
 #' @param formula an object of class "formula." Must have a survival object on
-#'   the left side and exactly two inputs on the right side: treatment and
-#'   historical. See "Details" for more information.
-#' @param data data frame. A data frame with columns 'time', 'status',
-#'   'treatment', and historical.' See "Details" for required structure.
+#'   the left side and at most one input on the right side, treatment. See
+#'   "Details" for more information.
+#' @param data a data frame containing the current data variables in the model.
+#'   Columns denoting 'time' and 'status' must be present. See "Details" for required
+#'   structure.
+#' @param data0 optional. A data frame containing the historical data variables in the model.
+#'   If present, the column labels of data and data0 must match.
 #' @param breaks vector. Breaks (interval starts) used to compose the breaks of the
 #'   piecewise exponential model. Do not include zero. Default breaks are the
 #'   quantiles of the input times.
@@ -24,7 +29,7 @@
 #'   \code{identity}. The discount function \code{scaledweibull} scales
 #'   the output of the Weibull CDF to have a max value of 1. The \code{identity}
 #'   discount function uses the posterior probability directly as the discount
-#'   weight. Default value is "\code{weibull}".
+#'   weight. Default value is "\code{identity}".
 #' @param alpha_max scalar. Maximum weight the discount function can apply.
 #'   Default is 1. For a two-arm trial, users may specify a vector of two values
 #'   where the first value is used to weight the historical treatment group and
@@ -44,9 +49,10 @@
 #'   historical treatment group and the second value is used to estimate the
 #'   weight of the historical control group.
 #' @param method character. Analysis method with respect to estimation of the weight
-#'   paramter alpha. Default value "\code{fixed}" estimates alpha once and holds it fixed
-#'   throughout the analysis. Alternative method "\code{mc}" estimates alpha for each
-#'   Monte Carlo iteration. See the the \code{bdpsurvival} vignette \cr
+#'   paramter alpha. Default method "\code{mc}" estimates alpha for each
+#'   Monte Carlo iteration. Alternate value "\code{fixed}" estimates alpha once
+#'   and holds it fixed throughout the analysis.  See the the
+#'   \code{bdpsurvival} vignette \cr
 #'   \code{vignette("bdpsurvival-vignette", package="bayesDP")} for more details.
 #' @param compare logical. Should a comparison object be included in the fit?
 #'   For a one-arm analysis, the comparison object is simply the posterior
@@ -57,10 +63,9 @@
 #'   \code{TRUE}.
 #' @details \code{bdpsurvival} uses a two-stage approach for determining the
 #'   strength of historical data in estimation of a survival probability outcome.
-#'   In the first stage, a Weibull distribution function is used as a
-#'   \emph{discount function} that defines the maximum strength of the
-#'   historical data (via \code{weibull_shape}, \code{weibull_scale}, and
-#'   \code{alpha_max}) and discounts based on disagreement with the current data.
+#'   In the first stage, a \emph{discount function} is used that
+#'   that defines the maximum strength of the
+#'   historical data and discounts based on disagreement with the current data.
 #'   Disagreement between current and historical data is determined by stochastically
 #'   comparing the respective posterior distributions under noninformative priors.
 #'   With a single arm survival data analysis, the comparison is the
@@ -68,30 +73,25 @@
 #'   survival. For a two-arm survival data, analysis the comparison is the
 #'   probability that the hazard ratio comparing treatment and control is
 #'   different from zero. The comparison metric \code{p} is then
-#'   input into the Weibull discount function and the final strength of the
+#'   input into the discount function and the final strength of the
 #'   historical data is returned (alpha).
 #'
 #'   In the second stage, posterior estimation is performed where the discount
-#'   function parameter, \code{alpha}, is used as a fixed value for all posterior
+#'   function parameter, \code{alpha}, is used incorporated in all posterior
 #'   estimation procedures.
 #'
 #'   To carry out a single arm (OPC) analysis, data for the current and
-#'   historical treatments are specified in a data frame. The data frame must have
-#'   columns with names 'time', 'status', 'treatment', and 'historical.' Column
-#'   'time' is the survival (censor) time of the event and 'status' is the
-#'   event indicator. The column 'treatment' is used to indicate which observations
-#'   are in the treatment and control group. A value of 1 indicates that the
-#'   observation is in the treatment group. The column 'historical' indicates
-#'   whether the observation is from the historical data (1) or current data (0).
-#'   The results are then based on the posterior probability of survival at
-#'   \code{surv_time} for the current data augmented by the historical data.
+#'   historical treatments are specified in separate data frames, data and data0,
+#'   respectively. The data frames must have matching columns denoting time and status.
+#'   The 'time' column is the survival (censor) time of the event and the 'status' column
+#'   is the event indicator. The results are then based on the posterior probability of
+#'   survival at \code{surv_time} for the current data augmented by the historical data.
 #'
 #'   Two-arm (RCT) analyses are specified similarly to a single arm trial. Again
-#'   the input data frame must have columns with names 'time', 'status',
-#'   'treatment', and 'historical.' Column 'time' is the survival (censor) time
-#'   of the event and 'status' is the event indicator. Now, the 'treatment'
-#'   column must use 0 to indicate the control group. The current data are
-#'   augmented by historical data (if present) and the results are then based
+#'   the input data frames must have columns denoting time and status, but now
+#'   an additional column named 'treatment' is required to denote treatment and control
+#'   data. The 'treatment' column must use 0 to indicate the control group. The current data
+#'   are augmented by historical data (if present) and the results are then based
 #'   on the posterior distribution of the hazard ratio between the treatment
 #'   and control groups.
 #'
@@ -177,51 +177,54 @@
 #'
 #' @examples
 #' # One-arm trial (OPC) example - survival probability at 5 years
-#' # Simulate survival data for a single arm (OPC) trial
-#' time   <- c(rexp(50, rate=1/20), rexp(50, rate=1/10))
-#' status <- c(rexp(50, rate=1/30), rexp(50, rate=1/30))
-#' status <- ifelse(time < status, 1, 0)
 #'
-#' # Collect data into a data frame
-#' example_surv_1arm <- data.frame(status     = status,
-#'                                 time       = time,
-#'                                 historical = c(rep(1,50),rep(0,50)),
-#'                                 treatment  = 1)
+#' # Collect data into data frames
+#' df_ <- data.frame(status = rexp(50, rate=1/30),
+#'                   time   = rexp(50, rate=1/20))
+#' df_$status <- ifelse(df_$time < df_$status, 1, 0)
 #'
-#' fit1 <- bdpsurvival(Surv(time, status) ~ historical + treatment,
-#'                     data = example_surv_1arm,
-#'                     surv_time = 5)
+#' df0 <- data.frame(status = rexp(50, rate=1/30),
+#'                   time   = rexp(50, rate=1/10))
+#' df0$status <- ifelse(df0$time < df0$status, 1, 0)
+#'
+#'
+#' fit1 <- bdpsurvival(Surv(time, status) ~ 1,
+#'                     data  = df_,
+#'                     data0 = df0,
+#'                     surv_time = 5,
+#'                     method = "fixed")
 #'
 #' print(fit1)
 #' \dontrun{
 #' plot(fit1)
 #' }
 #'
-#' # Two-arm trial (OPC) example
-#' # Simulate survival data for a two-arm trial
-#' time   <- c(rexp(50, rate=1/20), # Current treatment
-#'             rexp(50, rate=1/10), # Current control
-#'             rexp(50, rate=1/30), # Historical treatment
-#'             rexp(50, rate=1/5))  # Historical control
-#' status <- rexp(200, rate=1/40)
-#' status <- ifelse(time < status, 1, 0)
+#' # Two-arm trial example
+#' # Collect data into data frames
+#' df_ <- data.frame(time = c(rexp(50, rate=1/20),  # Current treatment
+#'                            rexp(50, rate=1/10)), # Current control
+#'                   status = rexp(100, rate=1/40),
+#'                   treatment = c(rep(1,50), rep(0,50)))
+#' df_$status <- ifelse(df_$time < df_$status, 1, 0)
 #'
-#' # Collect data into a data frame
-#' example_surv_2arm <- data.frame(status     = status,
-#'                                 time       = time,
-#'                                 historical = c(rep(0,100),rep(1,100)),
-#'                                 treatment  = c(rep(1,50),rep(0,50),rep(1,50),rep(0,50)))
-#'
-#' fit2 <- bdpsurvival(Surv(time, status) ~ historical + treatment,
-#'                        data = example_surv_2arm)
+#' df0 <- data.frame(time = c(rexp(50, rate=1/30),  # Historical treatment
+#'                            rexp(50, rate=1/5)),  # Historical control
+#'                   status =  rexp(100, rate=1/40),
+#'                   treatment = c(rep(1,50), rep(0,50)))
+#' df0$status <- ifelse(df0$time < df0$status, 1, 0)
 #'
+#' fit2 <- bdpsurvival(Surv(time, status) ~ treatment,
+#'                     data = df_,
+#'                     data0 = df0,
+#'                     method = "fixed")
 #' summary(fit2)
 #'
 #' ### Fix alpha at 1
-#' fit2_1 <- bdpsurvival(Surv(time, status) ~ historical + treatment,
-#'                       data = example_surv_2arm,
-#'                       fix_alpha = TRUE)
-#'
+#' fit2_1 <- bdpsurvival(Surv(time, status) ~ treatment,
+#'                       data = df_,
+#'                       data0 = df0,
+#'                       fix_alpha = TRUE,
+#'                       method = "fixed")
 #' summary(fit2_1)
 #'
 #'
@@ -238,17 +241,18 @@ bdpsurvival <- setClass("bdpsurvival", slots = c(posterior_treatment = "list",
 setGeneric("bdpsurvival",
   function(formula           = formula,
            data              = data,
+           data0             = NULL,
            breaks            = NULL,
            a0                = 0.1,
            b0                = 0.1,
            surv_time         = NULL,
-           discount_function = "weibull",
+           discount_function = "identity",
            alpha_max         = 1,
            fix_alpha         = FALSE,
            number_mcmc       = 10000,
            weibull_scale     = 0.135,
            weibull_shape     = 3,
-           method            = "fixed",
+           method            = "mc",
            compare           = TRUE){
              standardGeneric("bdpsurvival")
            })
@@ -257,37 +261,78 @@ setMethod("bdpsurvival",
   signature(),
   function(formula           = formula,
            data              = data,
+           data0             = NULL,
            breaks            = NULL,
            a0                = 0.1,
            b0                = 0.1,
            surv_time         = NULL,
-           discount_function = "weibull",
+           discount_function = "identity",
            alpha_max         = 1,
            fix_alpha         = FALSE,
            number_mcmc       = 10000,
            weibull_scale     = 0.135,
            weibull_shape     = 3,
-           method            = "fixed",
+           method            = "mc",
            compare           = TRUE){
 
+
+  ### Check validity of data input
+  call <- match.call()
+  if (missing(data)) {
+    stop("Current data not input correctly.")
+  }
+
+
+  ##############################################################################
+  ### Parse current data
+  ##############################################################################
   ### Check data frame and ensure it has the correct column names
-  namesData <- tolower(names(data))
-  namesDiff <- setdiff(c("status", "time", "historical", "treatment"), namesData)
-  if(length(namesDiff)>0){
-    nDiff <- length(namesDiff)
-    if(nDiff == 1){
-      errorMsg <- paste0("Column ",
-                         namesDiff,
-                         " is missing from the input data frame.")
-      stop(errorMsg)
-    } else if(nDiff>1){
-      errorNames <- paste0(namesDiff, collapse = ", ")
-      errorMsg <- paste0("Columns are missing from input data frame: ",
-                         errorNames)
-      stop(errorMsg)
+  mf <- mf0 <- match.call(expand.dots = FALSE)
+  m  <- match(c("formula", "data"), names(mf), 0L)
+  mf <- mf[c(1L, m)]
+  mf$na.action <- NULL
+  mf[[1L]] <- quote(stats::model.frame)
+  mf <- eval(mf, parent.frame())
+  mt <- attr(mf, "terms")
+  Y <- model.response(mf, "any")
+  if (length(dim(Y)) == 1L) {
+    nm <- rownames(Y)
+    dim(Y) <- NULL
+    if (!is.null(nm)) {
+      names(Y) <- nm
     }
   }
 
+  if(class(Y) != "Surv") stop("Data input incorrectly.")
+
+
+
+  ##############################################################################
+  ### Parse historical data
+  ##############################################################################
+  ### Parse historical data
+  if(!is.null(data0)){
+    m00 <- match("data0", names(mf0), 0L)
+    m01 <- match("data", names(mf0), 0L)
+    mf0[m01] <- mf0[m00]
+    m0  <- match(c("formula", "data"), names(mf0), 0L)
+    mf0 <- mf0[c(1L, m0)]
+    mf0$na.action <- NULL
+    mf0[[1L]] <- quote(stats::model.frame)
+    mf0 <- eval(mf0, parent.frame())
+    mt0 <- attr(mf0, "terms")
+    Y0 <- model.response(mf0, "any")
+    if (length(dim(Y0)) == 1L) {
+      nm0 <- rownames(Y0)
+      dim(Y0) <- NULL
+      if (!is.null(nm0)) {
+        names(Y0) <- nm0
+      }
+    }
+  } else{
+    Y0 <- NULL
+  }
+
 
   # Check that discount_function is input correctly
   all_functions <- c("weibull", "scaledweibull", "identity")
@@ -301,6 +346,7 @@ setMethod("bdpsurvival",
   treatment <- NULL
 
 
+
   ##############################################################################
   # Quick check, if alpha_max, weibull_scale, or weibull_shape have length 1,
   # repeat input twice
@@ -323,9 +369,10 @@ setMethod("bdpsurvival",
   ##############################################################################
   ### If no breaks input, create intervals along quantiles
   if(is.null(breaks)){
-     breaks <- quantile(data$time,probs=c(0.2,0.4,0.6,0.8))
+     breaks <- quantile(c(Y[,1], Y0[,1]),probs=c(0.2,0.4,0.6,0.8))
   }
 
+
   ### If zero is present in breaks, remove and give warning
   if(any(breaks==0)){
     breaks <- breaks[!(breaks==0)]
@@ -333,14 +380,47 @@ setMethod("bdpsurvival",
   }
 
 
+  ### Combine current and historical data, and format for analysis
+  dataCurrent <- data
+  dataCurrent$historical <- 0
+
+  if(!is.null(data0)){
+    dataHistorical            <- data0
+    dataHistorical$historical <- 1
+  } else{
+    dataHistorical <- NULL
+  }
+
+  dataALL <- rbind(dataCurrent, dataHistorical)
+
+
+  ### Update formula
+  formula <- update(formula, ~ . + historical)
 
   ### Split the data on the breaks
   dataSplit <- survSplit(formula,
                          cut     = breaks,
                          start   = "start",
                          episode = "interval",
-                         data    = data)
+                         data    = dataALL)
 
+  ### Change time and status column names
+  vars <- as.character(attr(mt, "variables"))[2]
+  vars <- strsplit(vars, "Surv\\(|, |\\)")[[1]]
+  var_time   <- vars[2]
+  var_status <- vars[3]
+
+  names(dataSplit)[match(var_time, names(dataSplit))] <- "time"
+  names(dataSplit)[match(var_status, names(dataSplit))] <- "status"
+
+
+  # Grab fu-time column
+  id_time <- names(dataSplit[ncol(dataSplit)-2])
+
+  # Look for treatment column, if missing, add it and set to 1
+  if(!any(names(dataSplit) == "treatment")){
+    dataSplit$treatment <- 1
+  }
 
   ### Compute exposure time within each interval
   dataSplit$exposure <- dataSplit$time - dataSplit$start
@@ -373,7 +453,7 @@ setMethod("bdpsurvival",
 
   ### If surv_time is null, replace with median time
   if(is.null(surv_time) & !arm2){
-    surv_time <- median(data$time)
+    surv_time <- median(c(Y[,1], Y[,0]))
   }
 
 
@@ -440,7 +520,8 @@ setMethod("bdpsurvival",
                 method            = method,
                 arm2              = arm2,
                 breaks            = breaks,
-                data              = data)
+                data              = dataSplit,
+                data_current      = data)
 
 
   ##############################################################################
@@ -643,7 +724,7 @@ posterior_survival <- function(S, S0, surv_time, discount_function,
           alpha_discount <- pweibull(p_hat, shape=weibull_shape,
                                      scale=weibull_scale)*alpha_max/max_p
         } else if(discount_function == "identity"){
-          alpha_discount <- p_hat
+          alpha_discount <- p_hat*alpha_max
         }
     }
   } else{
@@ -662,6 +743,7 @@ posterior_survival <- function(S, S0, surv_time, discount_function,
       V0     <- 1/apply(R0,2,var)
       logHR0 <- R0%*%V0/sum(V0)    #weighted average  of SE^2
       p_hat  <- mean(logHR0 > 0)   #larger is higher failure
+      p_hat  <- 2*ifelse(p_hat > 0.5, 1 - p_hat, p_hat)
     } else{
       stop("Unrecognized method. Use one of 'fixed' or 'mc'")
     }
@@ -669,7 +751,6 @@ posterior_survival <- function(S, S0, surv_time, discount_function,
     if(fix_alpha){
       alpha_discount <- alpha_max
     } else{
-        p_hat    <- 2*ifelse(p_hat > 0.5, 1 - p_hat, p_hat)
 
         # Compute alpha discount based on distribution
         if(discount_function == "weibull"){
@@ -681,7 +762,7 @@ posterior_survival <- function(S, S0, surv_time, discount_function,
           alpha_discount <- pweibull(p_hat, shape=weibull_shape,
                                      scale=weibull_scale)*alpha_max/max_p
         } else if(discount_function == "identity"){
-          alpha_discount <- p_hat
+          alpha_discount <- p_hat*alpha_max
         }
     }
   }
